@@ -36,7 +36,7 @@ glcm <- setClass("glcm",
 setMethod("initialize", 
           signature = "glcm", 
           definition = function(.Object, data, angle, d, n_grey, normalize, ...){
-
+            
             #Send to discretizeImage for error checking
             #Discretize grey values if required
             #discretize image and initialize GLCM based on discretized image
@@ -44,44 +44,79 @@ setMethod("initialize",
             
             unique_vals <- sort(unique(c(data)))
             
-            #Given an image matrix and angle, calculate glcm
-            #allows angles of 0 (0,1), 45 (-1,1), 90 (-1,0), 135 (-1,-1)
-            # d is distance
+            #If matrix is composed of a single value glcm is undefined
+            #return empty matrix
+            if(length(unique_vals) == 1 && length(which(data == unique_vals)) == 1){
+              .Object@.Data <- matrix(NA)[-1,-1]
+              return(.Object)
+            }
+            
+            #the value of 0 is reserved for NAs in the matrix, 
+            #if there are any 0's in the DF, add 1 to all values
+            #original values will be replaced after
+            if(is.element(0, data)) data <- data + 1
+            
+            #Convert All NAs to 0
+            data[is.na(data)] <- 0
+            
+            
             if(identical(angle, 0)){
-              angle <- c(0,1)*d
+              counts <- glcm0(data, n_grey = max(data), d)
+              
             } else if (identical(angle, 45)){
-              angle <- c(-1,1)*d
+              counts <- glcm45(data, n_grey = max(data), d)
+              
             } else if (identical(angle, 90)){
-              angle <- c(-1,0)*d
+              counts <- glcm90(data, n_grey = max(data), d)
+              
             } else if (identical(angle, 135)){
-              angle <- c(-1,-1)*d
+              counts <- glcm135(data, n_grey = max(data), d)
+              
             } else {
               stop("angle must be one of '0', '45', '90', '135'.")
             }
+                            
+            #Row 1 and Col 1 hold NA values, remove them
+            counts <- counts[-1, -1]
             
-            #define count matrix
-            counts <- matrix(0, ncol=length(unique_vals), nrow=length(unique_vals) )
-            rownames(counts) <- colnames(counts) <- unique_vals
-            
-            #ref = reference pixel value
-            #nei = neighbor pixel value
-            #This loop finds indices of neighbor pixels given a ref value and angle, counts occurrence of each
-            #pixel value, and adds them into the counts matrix
-            for(ref in unique_vals){
-              ref_indices <- which(data==ref, arr.ind=TRUE)
-              nei_indices <- matrix(c(ref_indices[,'row'] + angle[1], ref_indices[,'col'] + angle[2]), ncol=2)
-              #make sure indices exist
-              nei_indices <- matrix(nei_indices[which(nei_indices[,1] <= nrow(data) & nei_indices[,2] <= ncol(data)),], ncol=2)
-              nei <- data[nei_indices] 
-              nei_counts <- as.data.frame(table(nei))
-              counts[as.character(ref), as.character(nei_counts$nei) ] <- counts[as.character(ref), as.character(nei_counts$nei)] + nei_counts$Freq
+            #Situation where matrix is composed of a single NA
+            if(length(counts) == 0){
+              .Object@.Data <- counts
+              return(.Object)
+            }
+
+            #Replace proper values in column and row names
+            #Two situations:
+            #1. No zeroes were present, thus nothing was added
+            #2. One was added to all entries because there were zeros in the matrix
+            if(is.matrix(counts)){
+              if(dim(counts)[1] == max(unique_vals)){ #ie. 1 wasn't added
+                counts <- counts[unique_vals, unique_vals]
+                #counts <- counts[which(rownames(counts) %in% unique_vals), which(colnames(counts) %in% unique_vals)]
+              } else if (dim(counts)[1] == max(unique_vals)+1) {
+                #counts <- counts[which((as.numeric(rownames(counts)) - 1) %in% unique_vals), which((as.numeric(colnames(counts)) - 1) %in% unique_vals)]
+                counts <- counts[unique_vals + 1, unique_vals + 1]
+              }  
             }
             
+            if(!is.matrix(counts)) {
+              #Edge case where only a single grey value present - leads to a numeric, rather than a matrix
+              #Therefore case to 1x1 matrix
+              counts <- matrix(counts)
+            }
             
+            rownames(counts) <- colnames(counts) <- unique_vals
+                      
             #GLCMs should be symmetrical, so the transpose is added
             counts <- counts + t(counts)
             #Normalize
-            if(normalize) counts <- counts/sum(counts)
+            if(normalize){
+              count_sum <- sum(counts)
+              if(count_sum > 0){
+                counts <- counts/count_sum
+              }
+            }
+              
             
             
             .Object@.Data <- counts
@@ -92,5 +127,5 @@ setMethod("initialize",
 
 #' @export          
 glcm <- function(data, angle = 0, d=1, n_grey = 32, normalize=TRUE, ...){
- return(new("glcm", data, angle, d, n_grey, normalize, ...))
+  return(new("glcm", data, angle, d, n_grey, normalize, ...))
 }
